/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.example.services.impl;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.metatype.annotations.AttributeDefinition;
import org.osgi.service.metatype.annotations.Designate;
import org.osgi.service.metatype.annotations.ObjectClassDefinition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.example.services.HelloWorldService;

@Component(service = HelloWorldService.class)
@Designate(ocd = HelloWorldServiceImpl.Cfg.class)
public class HelloWorldServiceImpl implements HelloWorldService {
    private static final Logger log = LoggerFactory.getLogger(HelloWorldServiceImpl.class);

    private Cfg cfg;

    /* Service Methods */

    @Override
    public final String sayHello() {
        return String.format("Hello %s!", cfg.planet());
    }

    /* OSGi Component Methods */

    @Activate
    protected final void activate(final Cfg cfg) throws Exception {
        log.info("Activating Hello World Service with planet '{}'", cfg.planet());
        this.cfg = cfg;
    }

    @Deactivate
    protected final void deactivate(final Cfg cfg) {
    }

    @ObjectClassDefinition(name = "My Example - Hello World Service - Basic OSGi Service Configuration")
    public @interface Cfg {
        @AttributeDefinition(name = "Planet", description = "The planet from which to greet from.")
        String planet() default "Earth";
    }
}